@@ -247,7 +247,8 @@ def set_seed(seed=None):
 #%% Probabilities -- mostly not jitted since performance gain is minimal
 
 __all__ += ['n_binomial', 'binomial_filter', 'binomial_arr', 'n_multinomial',
-            'poisson', 'n_poisson', 'n_neg_binomial', 'choose', 'choose_r', 'choose_w']
+            'poisson', 'n_poisson', 'n_neg_binomial', 'choose', 'choose_r', 'choose_w',
+            'randround']
 
 def n_binomial(prob, n):
     '''
@@ -428,6 +429,33 @@ def choose_w(probs, n, unique=True):
     return np.random.choice(n_choices, n_samples, p=probs, replace=not(unique))
 
 
+def randround(x):
+    '''
+    Round a float, list, or array probabilistically to the nearest integer. Works
+    for both positive and negative values.
+
+    To move to Sciris eventually. Adapted from:
+        https://stackoverflow.com/questions/19045971/random-rounding-to-integer-in-python
+
+    Args:
+        x (int, list, arr): the floating point numbers to probabilistically convert to the nearest integer
+
+    Returns:
+        Array of integers
+
+    **Example**::
+
+        cv.randround(np.random.randn(20))
+    '''
+    if isinstance(x, np.ndarray):
+        output = np.array(np.floor(x+np.random.random(x.size)), dtype=int)
+    elif isinstance (x, list):
+        output = [randround(i) for i in x]
+    else:
+        output = int(np.floor(x+np.random.random()))
+    return output
+
+
 #%% Simple array operations
 
 __all__ += ['true',   'false',   'defined', 'undefined',
